import logging
import textwrap
from pathlib import Path

import owlrl
from owlrl import OWLRL_Extension
from rdflib import ConjunctiveGraph

logger = logging.getLogger(__name__)


def apply_inference_owlrl(graph: ConjunctiveGraph) -> None:
    """Apply OWL RL inference rules."""
    logger.info('Inference: OWL RL started...')
    owlrl.DeductiveClosure(OWLRL_Extension).expand(graph)
    logger.info('Inference: OWL RL complete.')


def apply_inference_sparql(
    inference_directory: Path,
    graph: ConjunctiveGraph,
) -> bool:
    """Apply custom SPARQL inference rules."""
    if inference_directory.is_dir():
        query_files = list(sorted(   # noqa: C413
            inference_directory.glob('**/*.sparql'),
        ))
    else:
        query_files = []

    if not query_files:
        logger.info(
            'No SPARQL inference files found at %s directory.',
            inference_directory,
        )
        return False

    logger.warning(
        'SPARQL inference queries are an outdated feature. Files: %s',
        query_files,
    )
    for query_file in query_files:
        logger.info(
            'Inference: %s',
            query_file.relative_to(inference_directory),
        )

        query_text = query_file.read_text()
        logger.debug('  Query text: %s', textwrap.indent(
            query_text,
            prefix='  ',
        ))

        graph.update(query_text)

    return True



import json
from io import StringIO
from typing import Iterable, Optional, TextIO

import yaml
from iolanta.convert_dollar_signs import convert_dollar_signs
from iolanta.loaders import Loader
from iolanta.models import LDContext, LDDocument, Quad
from iolanta.parsers.json import JSON
from rdflib import URIRef

try:  # noqa
    from yaml import CSafeLoader as SafeLoader  # noqa
except ImportError:
    from yaml import SafeLoader  # type: ignore   # noqa


class YAML(JSON):
    """Load YAML data."""

    def as_jsonld_document(self, raw_data: TextIO) -> LDDocument:
        """Read YAML content and adapt it to JSON-LD format."""
        document = yaml.load(raw_data, Loader=SafeLoader)
        return convert_dollar_signs(document)

    def as_quad_stream(
        self,
        raw_data: TextIO,
        iri: Optional[URIRef],
        context: LDContext,
        root_loader: Loader,
    ) -> Iterable[Quad]:
        """Read YAML-LD data into a quad stream."""
        json_data = self.as_jsonld_document(raw_data)

        return JSON().as_quad_stream(
            raw_data=StringIO(json.dumps(json_data, ensure_ascii=False)),
            iri=iri,
            context=context,
            root_loader=root_loader,
        )



import operator

from dominate.tags import div, h2, ul, li

from iolanta.facet import Facet
from iolanta.namespaces import IOLANTA
from octadocs.iolanta import render
from octadocs_ibis.models import IBIS


class SuggestedNextSteps(Facet):
    sparql_query = '''
    SELECT ?issue_page WHERE {
        $position ibis:suggests ?issue .
        ?issue octa:subjectOf* ?issue_page .
        ?issue_page a octa:Page .
    } ORDER BY ?issue
    '''

    def render(self):
        suggestions = map(
            operator.itemgetter('issue_page'),
            self.query(
                self.sparql_query,
                position=self.uriref,
            )
        )

        renderables = [
            render(
                suggestion,
                octiron=self.octiron,
                environments=[IOLANTA.html],
            )
            for suggestion in suggestions
        ]

        if renderables:
            list_items = list(map(li, renderables))

            return div(
                h2('Что дальше?'),
                ul(*list_items),
            )

        return ''



from pathlib import Path

from typer import Typer

app = Typer(name='context')


@app.callback(invoke_without_command=True)
def context(path: Path):
    """Print context for a path."""
    raise NotImplementedError('Not implemented yet.')




from pathlib import Path
from typing import Dict, Iterable, Type

from iolanta.loaders.local_file import Loader, LocalFile
from iolanta.loaders.named import NamedContextLoader
from iolanta.loaders.scheme_choice import SchemeChoiceLoader
from iolanta.models import LDContext, LDDocument, Quad
from rdflib import URIRef
from urlpath import URL


def choose_loader_by_url(url: URL) -> Type[Loader]:
    """Find loader by URL scheme."""
    return LocalFile


def as_document(url: URL) -> LDDocument:
    """Retrieve the document presented by the specified URL."""
    loader_class = choose_loader_by_url(url)
    return loader_class().as_jsonld_document(url)


def construct_root_loader(
    root_directory: Path,
    default_context: LDContext,
    named_contexts: Dict[str, LDContext],
) -> SchemeChoiceLoader:
    return SchemeChoiceLoader(
        loader_by_scheme={
            'file': LocalFile(
                root_directory=root_directory,
                default_context=default_context,
            ),
            'local': NamedContextLoader(
                named_contexts=named_contexts,
            ),
        },
    )


def as_quad_stream(
    url: URL,
    iri: URIRef,
    default_context: LDContext,
    root_directory: Path,
    named_contexts: Dict[str, LDContext],
) -> Iterable[Quad]:
    """Retrieve the stream presented by the specified URL."""
    root_loader = construct_root_loader(
        default_context=default_context,
        root_directory=root_directory,
        named_contexts=named_contexts,
    )

    return root_loader.as_quad_stream(
        url=url,
        iri=iri,
    )



from datetime import date, datetime
from typing import cast

from iolanta.facet import Facet
from rdflib import Literal


class DateLiteral(Facet):
    """Render a date."""

    def render(self):
        """Render date or datetime as a date."""
        literal = cast(Literal, self.iri)

        date_value = literal.value

        if isinstance(date_value, datetime):
            return str(date_value.date())

        if isinstance(date_value, date):
            return str(date_value)

        raise ValueError('Not a date! {}'.format(date_value))



import dataclasses
from pathlib import Path
from typing import Any

from documented import DocumentedError
from octadocs.types import QueryResult
from typing_extensions import Protocol


class QueryExecutor(Protocol):
    """
    Query executor.

    Allows to abstract away the details of SPARQL query execution from the
    StoredQuery interface.
    """

    def __call__(  # type: ignore
        self,
        query_text: str,
        **kwargs: Any,
    ) -> QueryResult:
        """Run given query with args against an RDF store."""


@dataclasses.dataclass
class QueryNotFound(DocumentedError):
    """Stored SPARQL query file {self.path} not found."""

    path: Path


@dataclasses.dataclass(frozen=True)
class StoredQuery:
    """
    Stored SPARQL query access interface.

    Accepts `executor`, which is a function that accepts query text and params,
    returning the query execution result.
    """

    path: Path
    executor: QueryExecutor

    def __call__(self, **kwargs) -> QueryResult:
        """Execute the query."""
        text = self._read_query_text()
        return self.executor(text, **kwargs)

    def _append(self, segment: str) -> 'StoredQuery':
        """Append another segment to the path."""
        return dataclasses.replace(
            self,
            path=self.path / segment,
        )

    def _read_query_text(self) -> str:
        """Fetch query text from disk."""
        query_path = self.path.with_name(f'{self.path.name}.sparql')

        try:
            return query_path.read_text()
        except FileNotFoundError as err:
            raise QueryNotFound(path=query_path) from err

    __getitem__ = __getattr__ = _append  # noqa: WPS429



import shelve
from dataclasses import dataclass
from pathlib import Path

from documented import DocumentedError
from rdflib import Graph


@dataclass
class CacheNotFound(DocumentedError):
    """
    Cache file not found.

    File Path: {self.file_path}

    To create the cache, please run `mkdocs serve` or `mkdocs build`.
    """

    file_path: Path


def save_graph(graph: Graph, path: Path):
    """Save graph to disk."""
    path.parent.mkdir(parents=True, exist_ok=True)
    with shelve.open(str(path)) as db:
        db['graph'] = graph


def load_graph(path: Path):
    """Load graph from disk."""
    with shelve.open(str(path)) as db:
        return db['graph']



import operator

from more_itertools import first
from octadocs.octiron import Octiron
from rdflib.term import Node


def default(octiron: Octiron, node: Node) -> str:
    """
    Default facet to render an arbitrary object.

    Relies upon rdfs:label.
    """
    rows = octiron.query(
        'SELECT ?label WHERE { ?node rdfs:label ?label }',
        node=node,
    )

    labels = map(
        operator.itemgetter('label'),
        rows,
    )

    return first(labels, '[No Label]')



from dominate.tags import div, p, h2
from more_itertools import first

from iolanta.facet import Facet
from iolanta.namespaces import IOLANTA
from octadocs.iolanta import render
from octadocs_ibis.models import IBIS
from octadocs_ibis.queries import retrieve_evaluation_for_position


class PositionEvaluation(Facet):
    sparql_query = '''
    SELECT ?evaluation ?value WHERE {
        $position ibis:evaluation ?evaluation .
        ?evaluation prov:value* ?value .

        FILTER (!isBlank(?value))
    }
    '''

    def render(self):
        try:
            is_approved, evaluation = retrieve_evaluation_for_position(
                position=self.uriref,
                ldflex=self.ldflex,
            )
        except TypeError:
            return ''

        if not evaluation:
            return ''

        admonition_class = 'success' if is_approved else 'failure'
        title = 'Принято' if is_approved else 'Отвергнуто'

        return div(
            h2('Оценка'),
            div(
                p(
                    title,
                    cls='admonition-title',
                ),
                p(
                    render(
                        evaluation,
                        environments=[IOLANTA.html],
                        octiron=self.octiron,
                    ),
                ),
                cls=f'admonition {admonition_class}',
            ),
        )



import functools
from typing import Iterable

from mkdocs.structure.nav import Navigation, Section
from mkdocs.structure.pages import Page
from octadocs.navigation.types import NavigationItem


@functools.singledispatch
def create_pages_list_by_navigation(
    navigation_item: NavigationItem,
) -> Iterable[Page]:
    """Filter pages."""
    raise NotImplementedError()


@create_pages_list_by_navigation.register(Page)
def _create_pages_list_by_page(page: Page) -> Iterable[Page]:
    yield page


@create_pages_list_by_navigation.register(Section)
def _create_pages_list_by_section(section: Section) -> Iterable[Page]:
    for navigation_item in section.children:
        yield from create_pages_list_by_navigation(navigation_item)


@create_pages_list_by_navigation.register(Navigation)
def _create_pages_list_by_navigation(navigation: Navigation) -> Iterable[Page]:
    for navigation_item in navigation.items:
        yield from create_pages_list_by_navigation(navigation_item)



import itertools
from datetime import date
from typing import Union

from classes import typeclass
from iolanta.cli.pretty_print import render_literal_value
from more_itertools import consume, first
from octadocs.types import QueryResult, SelectResult, ComputedQName
from rdflib import BNode, Graph, Literal, URIRef
from rich.console import Console
from rich.table import Table


@typeclass
def pretty_print_value(rdflib_value: Union[URIRef, Literal, BNode]) -> str:
    """Pretty print a value for a table in CLI."""


@pretty_print_value.instance(type(None))
def _pretty_print_none(none_value):
    """Format None."""
    return f'∅ {none_value}'


@pretty_print_value.instance(URIRef)
def _pretty_print_value_uri_ref(uriref: URIRef):
    """Format URI Ref."""
    return f'🔗 {uriref}'


@pretty_print_value.instance(ComputedQName)
def _pretty_print_value_uri_ref(qname: ComputedQName):
    """Format QName."""
    return (
        f'🔗 [link={qname.namespace_url}]'
        f'[blue]{qname.namespace_name}[/blue][/link]:'
        f'{qname.term}'
    )


@pretty_print_value.instance(Literal)
def _pretty_print_literal(literal: Literal):
    """Render a literal."""
    rendered_value = render_literal_value(literal.toPython())

    if literal.language:
        formatted_language = {
            'ru': '🇷🇺',
            'en': '🇺🇸',
            'ua': '🇺🇦',
        }[literal.language]

        return f'{formatted_language} {rendered_value}'

    return rendered_value


@pretty_print_value.instance(BNode)
def _pretty_print_bnode(bnode: BNode):
    """Print a blank node."""
    return f'😶 {bnode}'


@typeclass
def pretty_print(query_result: QueryResult):
    """Pretty print query result."""


@pretty_print.instance(SelectResult)
def _pretty_print_select_result(select_result: SelectResult):
    """Print a SPARQL query result in style."""
    if not select_result:
        return

    columns = first(select_result).keys()

    table = Table(
        *columns,
        show_header=True,
        header_style="bold magenta",
    )

    consume(itertools.starmap(
        table.add_row,
        [
            map(
                pretty_print_value,
                row.values(),
            )
            for row in select_result
        ],
    ))

    Console().print(table)


@pretty_print.instance(Graph)
def _pretty_construct(graph: Graph):
    if not graph:
        return

    table = Table(
        'Subject',
        'Predicate',
        'Object',
        show_header=True,
        header_style="bold magenta",
    )

    consume(itertools.starmap(
        table.add_row,
        [
            map(
                pretty_print_value,
                triple,
            )
            for triple in graph
        ],
    ))

    Console().print(table)



import json
import logging
import operator
import os
from concurrent.futures import ThreadPoolExecutor
from functools import partial
from pathlib import Path
from typing import Optional, List

import backoff
import rich
from github import (
    Github, UnknownObjectException, RateLimitExceededException,
)
from github.Repository import Repository
from typer import Typer
from urlpath import URL

from iolanta.models import LDDocument
from ldflex import LDFlex
from octadocs.storage import load_graph
from octadocs_github.models import GH

logger = logging.getLogger(__name__)
app = Typer(
    name='github',
    help='Manage data from GitHub.',
)

SPDX_URL = URL(
    'https://raw.githubusercontent.com/spdx/license-list-data/'
    'master/jsonld/licenses.jsonld',
)


def ldflex_from_cache() -> LDFlex:
    """Instantiate LDFLex from the cached graph."""
    graph = load_graph(Path.cwd() / '.cache/octadocs')
    return LDFlex(graph=graph)


GITHUB_URLS = '''
SELECT DISTINCT ?url WHERE {
    {
        ?url ?p ?o
    } UNION {
        ?s ?p ?url
    }

    FILTER isIRI(?url) .

    FILTER(
        STRSTARTS(
            str(?url),
            "https://github.com/"
        )
    ) .
}
'''


def extract_repo_name(url: URL) -> Optional[str]:
    """Extract repository name from a GitHub URL."""
    try:
        _hostname, owner_name, repo_name, *etc = url.parts
    except ValueError:
        return None

    return f'{owner_name}/{repo_name}'


@backoff.on_exception(backoff.expo, RateLimitExceededException)
def retrieve_releases(repo: Repository) -> List[LDDocument]:
    """Retrieve GitHub releases as a JSON document."""
    rich.print(f'Downloading: {repo.name} releases...')
    releases = repo.get_releases().reversed.get_page(0)
    return list(map(operator.attrgetter('raw_data'), releases))


@backoff.on_exception(backoff.expo, RateLimitExceededException)
def retrieve_repository_description(
    repository_name: str,
    github: Github,
) -> Optional[LDDocument]:
    """Retrieve JSON description of a GitHub repo."""
    rich.print(f'Downloading: {repository_name}')
    try:
        repo = github.get_repo(repository_name)
    except UnknownObjectException:
        rich.print(f'[red]Repo not found: [bold]{repository_name}[/bold][/red]')
        return

    description = repo.raw_data

    description.update({
        'releases': retrieve_releases(repo),
        '@id': description['html_url'],
    })

    # GitHub does not know the license, so we will not export it.
    try:
        spdx_id = description['license']['spdx_id']
    except TypeError:
        description.pop('license')
    else:
        if spdx_id == 'NOASSERTION':
            description.pop('license')

    return description


def download_repository(
    repository_name,
    github,
    target_dir,
):
    repository_description = retrieve_repository_description(
        repository_name=repository_name,
        github=github,
    )

    if repository_description is not None:
        file_name = repository_description['full_name'].replace('/', '__')
        (target_dir / f'{file_name}.json').write_text(json.dumps(
            repository_description,
            indent=2,
        ))


@app.command(name='update')
def github_cli():
    """Update GitHub information."""
    ldflex = ldflex_from_cache()
    uri_refs = map(
        operator.itemgetter('url'),
        ldflex.query(GITHUB_URLS),
    )

    urls = map(URL, uri_refs)

    repo_names = set(filter(bool, map(extract_repo_name, urls)))

    github = Github(os.getenv('GITHUB_TOKEN'))

    docs_dir = Path.cwd() / 'docs'

    if not docs_dir.is_dir():
        raise ValueError(
            f'{docs_dir} is considered to be docs directory but it does not '
            f'exist.',
        )

    target_dir = docs_dir / 'generated/octadocs-github'
    target_dir.mkdir(parents=True, exist_ok=True)
    (target_dir / 'context.json').write_text(json.dumps(
        {
            '@import': 'github-generated',

            # FIXME: See docs/decisions/0010-context-application-order.md
            '@vocab': str(GH),
        },
        indent=2,
    ))

    results = ThreadPoolExecutor(max_workers=5).map(
        partial(
            download_repository,
            github=github,
            target_dir=target_dir,
        ),
        repo_names,
    )

    for result in filter(bool, results):
        rich.print(result)



from typing import TypedDict

from rdflib import Namespace


class RepoDescription(TypedDict):
    """GitHub repo description from API."""

    stargazers_count: int


GH = Namespace('https://octadocs.io/github/')
SPDX = Namespace('http://spdx.org/licenses/')



from dataclasses import dataclass

from documented import DocumentedError
from dominate.tags import a, code
from iolanta.facet import Facet
from more_itertools import first
from octadocs_adr.models import ADRNumberInvalid
from rdflib.term import Node


@dataclass
class ADRNotFound(DocumentedError):
    """
    ADR was not found.

    !!! error "ADR not found"
        The page specified as {self.iri} was not found on this site.

        - Perhaps this page does not exist,
        - or it is not an ADR?

        Please check existence of this page and whether it is correctly
        described as an ADR, for example, has the required `number` property.
    """

    iri: Node


class LinkToADR(Facet):
    """Link to an ADR document."""

    sparql = '''
    SELECT * WHERE {
        ?page
            octa:url ?url ;
            octa:title ?label ;
            adr:number ?number .

        OPTIONAL {
            ?page adr:status / octa:symbol ?symbol .
        }
    } ORDER BY ?number LIMIT 1
    '''

    def render(self):
        """As an HTML link."""
        descriptions = self.query(
            self.sparql,
            page=self.iri,
        )

        try:
            location = first(descriptions)
        except ValueError as err:
            raise ADRNotFound(iri=self.iri) from err

        number = location['number'].value

        if not isinstance(number, int):
            raise ADRNumberInvalid(
                number=number,
                page=self.uriref,
            )

        readable_number = f'ADR{number:03}'

        if symbol := location.get('symbol'):
            readable_number = f'{symbol} {readable_number}'

        return a(
            code(readable_number),
            ' ',
            location['label'],
            href=location['url'],
        )



from dataclasses import dataclass, field
from typing import Dict, Optional

from documented import DocumentedError
from iolanta.loaders import Loader
from iolanta.loaders.base import PyLDOptions, PyLDResponse
from iolanta.models import LDContext, LDDocument
from rdflib import URIRef
from urlpath import URL


@dataclass
class NamedContextNotFound(DocumentedError):
    """
    Named context not found.

        URL: {self.url}
        Supported contexts: {self.context_names}
    """

    url: str
    named_contexts: Dict[str, LDContext]

    @property
    def context_names(self):
        """Render names of supported contexts."""
        return ', '.join(self.named_contexts.keys())


@dataclass(frozen=True)
class NamedContextLoader(Loader):
    """Retrieve context by name."""

    named_contexts: Dict[str, LDContext] = field(repr=False)

    def construct_context_name(self, url: URL) -> str:
        """Clean the context name."""
        return url.path

    def as_jsonld_document(
        self,
        url: URL,
        iri: Optional[URIRef] = None,
    ) -> LDDocument:
        """Get JSON-LD document from named contexts."""
        context_name = self.construct_context_name(url)

        try:
            return self.named_contexts[context_name]
        except KeyError:
            raise NamedContextNotFound(
                url=url,
                named_contexts=self.named_contexts,
            )

    def __call__(self, url: str, options: PyLDOptions) -> PyLDResponse:
        """Retrieve the pre loaded named context by name."""
        return {
            'document': self.as_jsonld_document(
                url=url,
                iri=url,
            ),
            'contextUrl': None,
        }



import sys
from pathlib import Path
from typing import Optional

from typer import Argument, Option, Typer

from ldflex import LDFlex
from octadocs.cli.formatters.choose import cli_print
from octadocs.storage import load_graph
from octadocs.types import QueryResultsFormat

app = Typer(name='sparql')


@app.callback(invoke_without_command=True)
def sparql(
    fmt: QueryResultsFormat = Option(
        default=QueryResultsFormat.PRETTY,
        metavar='format',
    ),
    query_text: Optional[str] = Argument(
        None,
        metavar='query',
        help='SPARQL query text. Will be read from stdin if empty.',
    ),
    use_qnames: bool = Option(
        default=True,
        help='Collapse URLs into QNames.',
    ),
) -> None:
    """Run a SPARQL query against the graph."""
    if query_text is None:
        query_text = sys.stdin.read()

    graph = load_graph(Path.cwd() / '.cache/octadocs')
    ldflex = LDFlex(graph)

    query_result = ldflex.query(query_text)

    cli_print(
        query_result=query_result,
        output_format=fmt,
        display_iri_as_qname=use_qnames,
        graph=graph,
    )



import rdflib
from iolanta.models import Triple


def reformat_blank_nodes(prefix: str, triple: Triple) -> Triple:
    """
    Prepend prefix to every blank node.

    JSON-LD flattening creates sequential identifiers in the form of _:bN
    for every blank node. See https://www.w3.org/TR/json-ld11/

    This means that subgraphs of the Octiron ConjunctiveGraph have clashing
    blank node identifiers.

    To avoid that, we prepend a prefix to every blank node identifier.
    """
    return Triple(*(
        rdflib.BNode(value=prefix + str(singleton)) if (
            isinstance(singleton, rdflib.BNode)
        ) else singleton
        for singleton in triple
    ))



from typing import Protocol

from octadocs.octiron import Octiron
from rdflib import URIRef
from rdflib.term import Node


class PythonFacet(Protocol):
    """Prototype of a Python facet function."""

    def __call__(
        self,
        octiron: Octiron,
        node: Node,
        environment: URIRef,
    ) -> str:
        ...



import operator
from dataclasses import dataclass
from typing import Any, Dict, Iterable, List

from dominate.tags import table, tbody, td, th, thead, tr
from dominate.util import raw
from iolanta.facet import Facet
from octadocs.iolanta import HTML, render
from octadocs.octiron import Octiron
from octadocs_table.models import TABLE
from rdflib import URIRef

Row = Dict[URIRef, Any]   # type: ignore


def list_columns(
    iri: URIRef,
    octiron: Octiron,
) -> List[URIRef]:
    """List of column IRIs for a table."""
    # Idea: http://www.snee.com/bobdc.blog/2014/04/rdf-lists-and-sparql.html
    return list(
        map(
            operator.itemgetter('column'),
            octiron.query(
                query_text='''
                SELECT ?column WHERE {
                    ?iri table:columns/rdf:rest*/rdf:first ?column .
                }
                ''',
                iri=URIRef(iri),
            ),
        ),
    )


def construct_headers(
    octiron: Octiron,
    table_iri: URIRef,
    columns: List[URIRef],
) -> Iterable[th]:
    """Construct table headers."""
    return (
        th(
            render(
                node=column,
                octiron=octiron,
                environments=[table_iri, TABLE.thead, HTML],
            ),
        ) for column in columns
    )


def construct_row(
    instance: URIRef,
    octiron: Octiron,
    columns: List[URIRef],
) -> Row:
    """Construct a table row."""
    formatted_columns = '({columns})'.format(
        columns=', '.join([
            f'<{column}>' for column in columns
        ]),
    )

    query_text = '''
    SELECT * WHERE {
        $instance ?column ?value .

        OPTIONAL {
            ?value octa:trustLevel ?trust_level .
        }

        FILTER(?column IN %s) .
    } ORDER BY ?column ?trust_level
    ''' % formatted_columns

    cells = octiron.query(
        query_text=query_text,
        instance=instance,
    )

    cells.append({
        'column': TABLE.self,
        'value': instance,
    })

    # This dictionary comprehension entails an implicit deduplication by
    # `cell['column']`, in which the last duplicate wins. Since we have sorted
    # the elements by `octa:trustLevel` this means we will prefer a higher trust
    # level over a lower one, or over an absence of defined trust level.
    return {
        cell['column']: instance if (
            cell['column'] == TABLE.self
        ) else cell['value']
        for cell in cells
    }


def select_instances(
    iri: URIRef,
    octiron: Octiron,
) -> Iterable[URIRef]:
    """Select instances, or rows, for the table."""
    return map(
        operator.itemgetter('instance'),
        octiron.query(
            query_text='''
            SELECT ?instance WHERE {
                $iri table:class ?class .
                ?instance a ?class .
            }
            ''',
            iri=iri,
        ),
    )


def render_row(
    row: Row,
    columns: List[URIRef],
    octiron: Octiron,
) -> Iterable[td]:
    """Compile a sequence of table cells for a row."""
    for column in columns:
        try:
            cell_value = row[column]
        except KeyError:
            yield td()
            continue

        cell_content = str(
            render(
                node=cell_value,
                octiron=octiron,
                environments=[column, TABLE.td, HTML],
            ),
        )
        yield td(raw(cell_content))


def get_ordering(iri: URIRef, octiron: Octiron) -> List[URIRef]:
    """List of columns that we are ordering by."""
    # Idea: http://www.snee.com/bobdc.blog/2014/04/rdf-lists-and-sparql.html
    return list(
        map(
            operator.itemgetter('column'),
            octiron.query(
                query_text='''
                SELECT ?column WHERE {
                    ?iri table:ordering/rdf:rest*/rdf:first ?column .
                }
                ''',
                iri=URIRef(iri),
            ),
        ),
    )


def construct_sorter(ordering: List[URIRef]):
    """Construct a sorting procedure for rows in a table."""
    def sorter(row: Row):   # noqa: WPS430
        return [
            row.get(order_field, None)
            for order_field in ordering
        ]

    return sorter


def order_rows(
    rows: List[Row],
    ordering: List[URIRef],
):
    """Order rows by particular properties."""
    return sorted(
        rows,
        key=construct_sorter(ordering),
    )


@dataclass
class Table(Facet):
    """Octadocs Table."""

    def render(self) -> table:
        """Render the table."""
        columns = list_columns(
            iri=self.uriref,
            octiron=self.octiron,
        )

        ordering = get_ordering(
            iri=self.uriref,
            octiron=self.octiron,
        )

        headers = construct_headers(
            octiron=self.octiron,
            table_iri=self.uriref,
            columns=columns,
        )

        instances = select_instances(
            iri=self.uriref,
            octiron=self.octiron,
        )

        rows = [
            construct_row(
                instance=instance,
                columns=columns,
                octiron=self.octiron,
            )
            for instance in instances
        ]

        rows = order_rows(
            rows=rows,
            ordering=ordering,
        )

        rows = [
            tr(
                *render_row(
                    row=row,
                    columns=columns,
                    octiron=self.octiron,
                ),
            )
            for row in rows
        ]

        return table(
            thead(
                tr(*headers),
            ),
            tbody(*rows),
        )



from dominate.tags import script
from urlpath import URL

from iolanta.facet import Facet


class TelegramLink(Facet):
    """Embed a Telegram post by link."""

    def render(self):
        url = URL(str(self.iri))

        return script(
            _async=True,
            src='https://telegram.org/js/telegram-widget.js?19',
            data_telegram_post=url.path,
            data_width='100%',
        )



from typing import Any, Dict, List, NamedTuple, Union

from rdflib import Literal, URIRef
# JSON-LD context
from rdflib.term import Node

LDContext = Dict[str, Any]  # type: ignore
LDDocument = Union[Dict[str, Any], List[Dict[str, Any]]]  # type: ignore


# Named context URLs
ContextAliases = Dict[str, str]


def render_node(node: Node) -> str:
    """Render an RDFLib node as string."""
    if isinstance(node, URIRef):
        return f'<{node}>'

    if isinstance(node, Literal):
        rendered_node = f'"{node}"'
        if node.datatype:
            rendered_node = f'{rendered_node}^^{node.datatype}'

        if node.language:
            rendered_node = f'{rendered_node}@{node.language}'

        return rendered_node

    return f'<??? What is this? {node} ???>'


class Triple(NamedTuple):
    """RDF triple."""

    subject: URIRef
    predicate: URIRef
    object: Union[URIRef, Literal]  # noqa: WPS125

    def as_quad(self, graph: URIRef) -> 'Quad':
        """Add graph to this triple and hence get a quad."""
        return Quad(
            subject=self.subject,
            predicate=self.predicate,
            object=self.object,
            graph=graph,
        )


class Quad(NamedTuple):
    """Triple assigned to a named graph."""

    subject: Node
    predicate: Node
    object: Node  # noqa: WPS125
    graph: URIRef

    def as_triple(self):
        """Convert this to triple."""
        return Triple(self.subject, self.predicate, self.object)

    def __repr__(self):
        (
            rendered_subject,
            rendered_predicate,
            rendered_object,
            rendered_graph,
        ) = map(render_node, self)
        return (
            f'({rendered_subject} {rendered_predicate} {rendered_object} @ '
            f'{rendered_graph})'
        )



from dominate.tags import p, span
from dominate.util import raw, text
from more_itertools import first

from iolanta.facet import Facet
from iolanta.namespaces import IOLANTA
from octadocs.iolanta import render
from octadocs_ibis.models import IBIS


class Argument(Facet):
    sparql_query = '''
        SELECT ?agent WHERE {
            $argument ibis:endorsed-by ?agent .
        }
    '''

    def render(self):
        rows = self.query(self.sparql_query, argument=self.uriref)

        argument_body = render(
            self.uriref,
            environments=[IOLANTA.html],
            octiron=self.octiron,
        )

        try:
            row = first(rows)

        except ValueError:
            return argument_body

        agent = render(
            row['agent'],
            environments=[IOLANTA.html, IOLANTA.td],
            octiron=self.octiron,
        )

        return span(
            p(
                agent,
                text(': '),
            ),
            argument_body
        )




from abc import ABC
from dataclasses import dataclass
from typing import Iterable, Optional, TextIO

from iolanta.loaders.base import Loader
from iolanta.models import LDContext, LDDocument, Quad
from rdflib import URIRef


@dataclass(frozen=True)
class Parser(ABC):
    """
    Parser reads data from a file-like object and interprets them.

    For interpretation, it is also supplied with a context.
    """

    blank_node_prefix: str = ''

    def as_jsonld_document(
        self,
        raw_data: TextIO,
    ) -> LDDocument:
        """Generate a JSON-LD document."""
        raise NotImplementedError(
            f'{self}.as_json_document() is not implemented.',
        )

    def as_quad_stream(
        self,
        raw_data: TextIO,
        iri: Optional[URIRef],
        context: LDContext,
        root_loader: Loader,
    ) -> Iterable[Quad]:
        raise NotImplementedError(
            f'{self}.as_quad_stream() is not implemented.',
        )



from typing import Optional, Tuple

from more_itertools import first
from rdflib import URIRef
from rdflib.term import Node

from ldflex import LDFlex

EVALUATION_QUERY = '''
    SELECT ?evaluation ?value WHERE {
        $position ibis:evaluation ?evaluation .
        ?evaluation prov:value* ?value .

        FILTER (isLiteral(?value))
    }
'''


def retrieve_evaluation_for_position(
    position: URIRef,
    ldflex: LDFlex,
) -> Optional[Tuple[Optional[bool], Optional[Node]]]:
    """Find out whether the position is accepted by us or not."""
    rows = ldflex.query(EVALUATION_QUERY, position=position)

    try:
        row = first(rows)
    except ValueError:
        return None, None

    return row['value'].value, row['evaluation']



from copy import deepcopy

from deepmerge import always_merger
from octadocs.types import Context


def merge(first: Context, second: Context) -> Context:
    """
    Merge two contexts into one.

    Second context can override the first.
    """
    if isinstance(second, list):
        return [
            merge(first, sub_second)
            for sub_second in second
        ]

    return deepcopy(always_merger.merge(
        base=first,
        nxt=second,
    ))



import itertools
from dataclasses import dataclass
from typing import Tuple, Iterable, Optional, Type, Union, Set

from boltons.iterutils import remap
from more_itertools import first, last
from rdflib import Graph, URIRef, RDFS, BNode, OWL, Namespace, RDF
from rdflib.namespace import DefinedNamespace

from autocontext.models import AUTO
from iolanta.models import LDContext


@dataclass
class AutoContextBuilder:
    """Build JSON-LD context for a graph."""

    graph: Graph
    namespace: Optional[Union[Type[DefinedNamespace], URIRef]] = None
    exclude: Optional[Set[URIRef]] = None

    def build(self) -> LDContext:
        """Construct context."""
        rules = dict(self.construct_rules())
        shortcut_rules = dict(self.construct_shortcut_rules(rules))
        return self.add_namespace_rules({
            **rules,
            **shortcut_rules,
        })

    def compact(self, iri: URIRef) -> Tuple[Optional[str], Optional[str], str]:
        """Create a compact form of an IRI."""
        qname = self.graph.compute_qname(iri)
        try:
            namespace_name, namespace_iri, term = qname
        except ValueError:
            return None, None, str(iri)

        return namespace_name, namespace_iri, f'{namespace_name}:{term}'

    def range_by_property(self) -> Iterable[Tuple[URIRef, URIRef]]:
        range_triples = self.graph.triples((None, RDFS.range, None))

        range_pairs = [
            (property_type, value_type)
            for property_type, _range, value_type
            in range_triples
        ]

        grouped_range_pairs = itertools.groupby(
            sorted(
                range_pairs,
                key=first,
            ),
            key=first,
        )

        range_types_by_property = {
            property_type: set(map(last, pairs))
            for property_type, pairs in grouped_range_pairs
        }

        node_types = {OWL.Thing, RDFS.Resource}
        for property_type, range_types in range_types_by_property.items():
            if self.namespace and not property_type.startswith(self.namespace):
                continue

            if self.exclude and property_type in self.exclude:
                continue

            if range_types & node_types:
                yield property_type, {
                    '@type': '@id',
                }

    def construct_rules(self):
        yield from self.range_by_property()

    def construct_shortcut_rules(self, rules: LDContext):
        for term, _has_shortcut, shortcut in self.graph.triples((None, AUTO.shortcut, None)):
            yield str(shortcut), {
                '@id': term,
                **rules.get(term, {}),
            }

    def add_namespace_rules(self, rules: LDContext) -> LDContext:
        """
        Compact all URIRef instances in the context, converting them to
        QName strings, and add declarations for namespaces which those
        strings are referencing.

        This function is quite dirty. I did not yet find a way
        to make it better.
        """
        namespaces = {}

        def context_post_processor(path, key, value):
            if isinstance(key, URIRef):
                key_ns_name, key_ns_iri, key = self.compact(key)

                if key_ns_name:
                    namespaces[key_ns_name] = key_ns_iri

            if isinstance(value, URIRef):
                value_ns_name, value_ns_iri, value = self.compact(value)

                if value_ns_name:
                    namespaces[value_ns_name] = value_ns_iri

            return key, value

        rules = remap(rules, context_post_processor)

        return {
            **namespaces,
            **rules,
        }



import json
import re
from io import StringIO
from typing import Iterable, Optional, TextIO

import frontmatter
from yaml.scanner import ScannerError

from iolanta.convert_dollar_signs import convert_dollar_signs
from iolanta.loaders import Loader
from iolanta.models import LDContext, LDDocument, Quad
from iolanta.parsers.errors import YAMLError
from iolanta.parsers.json import JSON, assign_key_if_not_present
from iolanta.parsers.yaml import YAML
from octadocs.types import OCTA
from rdflib import RDF, URIRef

try:  # noqa
    from yaml import CSafeLoader as SafeLoader  # noqa
except ImportError:
    from yaml import SafeLoader  # type: ignore   # noqa


class Markdown(YAML):
    """Load YAML data."""

    def as_jsonld_document(self, raw_data: TextIO) -> LDDocument:
        """Read YAML content and adapt it to JSON-LD format."""
        raw_data.seek(0)
        document = frontmatter.load(raw_data).metadata
        return convert_dollar_signs(document)

    def construct_page_url(self, iri: URIRef) -> str:
        """Construct URL of a page."""
        match = re.match('^local:(?P<path>.+)$', str(iri))
        if match is None:
            raise ValueError(f'Could not decode URL: {iri}')

        url = match.groupdict()['path']

        url = re.sub(
            r'(index)?\.md$',
            '',
            url,
        )

        if not url.endswith('/'):
            url = f'{url}/'

        return f'/{url}'

    def as_quad_stream(
        self,
        raw_data: TextIO,
        iri: Optional[URIRef],
        context: LDContext,
        root_loader: Loader,
    ) -> Iterable[Quad]:
        """Assign octa:url and generate quad stream."""
        try:
            json_data = self.as_jsonld_document(raw_data)
        except ScannerError as err:
            raise YAMLError(
                iri=iri,
                error=err,
            ) from err

        json_data = assign_key_if_not_present(
            document=json_data,
            key=str(OCTA.url),
            default_value=self.construct_page_url(iri),
        )

        quad_stream = JSON().as_quad_stream(
            raw_data=StringIO(json.dumps(json_data, ensure_ascii=False)),
            iri=iri,
            context=context,
            root_loader=root_loader,
        )

        return [
            Quad(iri, RDF.type, OCTA.Page, iri),
            *quad_stream,
        ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['autocontext',
 'iolanta',
 'iolanta.cli',
 'iolanta.facets',
 'iolanta.loaders',
 'iolanta.parsers',
 'ldflex',
 'octadocs',
 'octadocs.cli',
 'octadocs.cli.formatters',
 'octadocs.facets',
 'octadocs.iolanta',
 'octadocs.navigation',
 'octadocs.octiron',
 'octadocs.octiron.plugins',
 'octadocs.pyld_document_loaders',
 'octadocs_adr',
 'octadocs_adr.facets',
 'octadocs_github',
 'octadocs_github.facets',
 'octadocs_ibis',
 'octadocs_ibis.facets',
 'octadocs_prov',
 'octadocs_prov.facets',
 'octadocs_table',
 'octadocs_table.facets',
 'octadocs_telegram',
 'octadocs_telegram.facets']

package_data = \
{'': ['*'],
 'iolanta.facets': ['sparql/*'],
 'octadocs': ['data/foaf/*',
              'data/iolanta/*',
              'data/octa/*',
              'data/owl/*',
              'data/rdf/*',
              'data/rdfs/*',
              'data/skos/*',
              'yaml/*'],
 'octadocs_adr': ['templates/octadocs-decisions/*', 'yaml/*'],
 'octadocs_github': ['data/*'],
 'octadocs_ibis': ['data/*', 'templates/octadocs-ibis/*'],
 'octadocs_prov': ['data/*'],
 'octadocs_table': ['templates/octadocs-table/*', 'yaml/*'],
 'octadocs_table.facets': ['sparql/*'],
 'octadocs_telegram': ['data/*']}

install_requires = \
['PyGithub>=1.55,<2.0',
 'PyLD>=2.0.3,<3.0.0',
 'backoff>=1.11.1,<2.0.0',
 'backports.cached-property>=1.0.0,<2.0.0',
 'boltons>=21.0.0,<22.0.0',
 'classes>=0.4.0,<0.5.0',
 'deepmerge>=0.1.1,<0.2.0',
 'documented>=0.1.1,<0.2.0',
 'dominate>=2.6.0,<3.0.0',
 'funcy>=1.17,<2.0',
 'graphviz>=0.17,<0.18',
 'mkdocs-macros-plugin>=0.7.0,<0.8.0',
 'mkdocs>=1.3.0,<2.0.0',
 'more-itertools>=8.9.0,<9.0.0',
 'owlrl>=6.0.2,<7.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'pydotplus>=2.0.2,<3.0.0',
 'pygraphviz>=1.9,<2.0',
 'python-frontmatter>=0.5.0,<0.6.0',
 'rdflib-jsonld>=0.6.1,<0.7.0',
 'rdflib>=6.1.1,<7.0.0',
 'requests>=2.25.1,<3.0.0',
 'rich>=10.3.0,<11.0.0',
 'singledispatchmethod>=1.0,<2.0',
 'typer>=0.3.2,<0.4.0',
 'typing-extensions>=3.7.4,<4.0.0',
 'urlpath>=1.1.7,<2.0.0']

entry_points = \
{'console_scripts': ['octadocs = octadocs.cli:app'],
 'mkdocs.plugins': ['octadocs = octadocs.plugin:OctaDocsPlugin',
                    'octadocs-github = octadocs_github.plugin:GithubPlugin',
                    'octadocs-ibis = octadocs_ibis.plugin:IbisPlugin',
                    'octadocs-prov = octadocs_prov.plugin:ProvenancePlugin',
                    'octadocs-table = octadocs_table.plugin:TablePlugin',
                    'octadocs-telegram = '
                    'octadocs_telegram.plugin:TelegramPlugin',
                    'octadocs_adr = octadocs_adr.plugin:ADRPlugin']}

setup_kwargs = {
    'name': 'octadocs',
    'version': '0.2.27',
    'description': 'MkDocs wiki made smart',
    'long_description': '# octadocs\n\n[![Build Status](https://travis-ci.com/anatoly-scherbakov/octadocs.svg?branch=master)](https://travis-ci.com/anatoly-scherbakov/octadocs)\n[![Coverage](https://coveralls.io/repos/github/anatoly-scherbakov/octadocs/badge.svg?branch=master)](https://coveralls.io/github/anatoly-scherbakov/octadocs?branch=master)\n[![Python Version](https://img.shields.io/pypi/pyversions/octadocs.svg)](https://pypi.org/project/octadocs/)\n[![wemake-python-styleguide](https://img.shields.io/badge/style-wemake-000000.svg)](https://github.com/wemake-services/wemake-python-styleguide)\n\n## Features\n\n- Fully typed with annotations and checked with mypy, [PEP561 compatible](https://www.python.org/dev/peps/pep-0561/)\n- Add yours!\n\n\n## Installation\n\n```bash\npip install octadocs\n```\n\n## License\n\n[MIT](https://github.com/anatoly-scherbakov/octadocs/blob/master/LICENSE)\n\n\n## Credits\n\nThis project was generated with [`wemake-python-package`](https://github.com/wemake-services/wemake-python-package). Current template version is: [868260c2d659e455bafc2ed4fe242413ef39e4dc](https://github.com/wemake-services/wemake-python-package/tree/868260c2d659e455bafc2ed4fe242413ef39e4dc). See what is [updated](https://github.com/wemake-services/wemake-python-package/compare/868260c2d659e455bafc2ed4fe242413ef39e4dc...master) since then.\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/octadocs/octadocs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.10',
}


setup(**setup_kwargs)



from typing import Dict, TypeVar, Iterable

import funcy
from dominate.tags import i
from funcy import first
from urlpath import URL

from iolanta.facet import Facet
from iolanta.namespaces import IOLANTA
from octadocs.environment import iri_to_url
from octadocs.iolanta import render


Something = TypeVar('Something')


def path_depth(row: Dict[str, str]) -> int:
    """Depth of path."""
    url = URL(iri_to_url(row['parent_directory_index']))
    return len(url.parts)


def but_first(iterable: Iterable[Something]) -> Iterable[Something]:
    """Skip the first item of an iterable and return the remaining."""
    try:
        next(iterable)
    except StopIteration:
        pass

    return iterable


class Breadcrumbs(Facet):
    breadcrumbs_query = '''
    SELECT ?parent_directory_index ?parent_directory_index_subject WHERE {
        $this octa:subjectOf* ?this_page .

        ?this_page octa:isChildOf* ?parent_directory .
        ?parent_directory octa:isParentOf ?parent_directory_index .
        ?parent_directory_index a octa:IndexPage .

        ?parent_directory_index_subject octa:subjectOf ?parent_directory_index .

        FILTER( ?parent_directory_index != ?this_page )
    } ORDER BY ?parent_directory_index
    '''

    page_type_query = '''
    SELECT ?type WHERE {
        $this a ?type .
        ?type octa:position ?position .
    } ORDER BY ?position
    '''

    def renderable_page_type(self):
        return first(
            self.query(
                self.page_type_query,
                this=self.uriref,
            ),
        )['type']

    def render(self):
        rows = self.query(
            self.breadcrumbs_query,
            this=self.uriref,
        )

        sorted_rows = sorted(
            rows,
            key=path_depth,
        )

        instances = but_first(
            funcy.pluck(
                'parent_directory_index_subject',
                sorted_rows,
            ),
        )

        rendered_links = [
            render(
                instance,
                octiron=self.octiron,
                environments=[IOLANTA.html],
            )
            for instance in instances
        ]

        breadcrumbs = ' › '.join(map(str, rendered_links))

        rendered_type = i(
            render(
                self.renderable_page_type(),
                octiron=self.octiron,
                environments=[IOLANTA.html],
            ),
        )

        return ' › '.join(
            filter(
                bool,
                map(
                    str,
                    [
                        breadcrumbs,
                        rendered_type,
                    ],
                ),
            ),
        )



import json
from functools import lru_cache
from pathlib import Path

import yaml
from iolanta.convert_dollar_signs import convert_dollar_signs
from octadocs.types import Context


@lru_cache(maxsize=None)
def context_from_json(path: Path) -> Context:
    """Load context.json file and return its content."""
    with path.open('r') as context_file:
        return json.load(context_file)


@lru_cache(maxsize=None)
def context_from_yaml(path: Path) -> Context:
    """Load context.json file and return its content."""
    with path.open('r') as context_file:
        raw_data = yaml.load(context_file, Loader=yaml.SafeLoader)
        return convert_dollar_signs(raw_data)



from pathlib import Path
from typing import Dict, Any

from mkdocs.plugins import BasePlugin
from mkdocs.structure.pages import Page
from rdflib import URIRef
from urlpath import URL

from iolanta import as_document
from iolanta.models import LDContext
from octadocs.mixins import OctadocsMixin
from octadocs_telegram.models import TELEGRAM

import logging

logger = logging.getLogger(__name__)


class TelegramPlugin(OctadocsMixin, BasePlugin):
    """Telegram links plugins."""

    plugin_data_dir = Path(__file__).parent / 'data'
    sparql_query = '''
    PREFIX telegram: <%s>

    INSERT {
        ?url a telegram:TelegramLink .
    } WHERE {
        { ?url ?p ?o . } UNION { ?s ?p ?url }

        FILTER(
            STRSTARTS(
                str(?url),
                "https://t.me/"
            )
        ) .
    }
    ''' % TELEGRAM

    def named_contexts(self) -> Dict[str, LDContext]:
        """Reusable named contexts."""
        return {
            'telegram': as_document(
                URL('file://', self.plugin_data_dir / 'named-context.yaml'),
            ),
        }

    def vocabularies(self) -> Dict[URIRef, Path]:
        """Load IBIS ontology."""
        return {
            URIRef(TELEGRAM): self.plugin_data_dir / 'telegram.yaml',
        }

    @property
    def templates_path(self) -> Path:
        """Templates associated with the plugin."""
        return Path(__file__).parent / 'templates'

    def on_page_context(
        self,
        context: Dict[str, Any],
        page: Page,
        **kwargs,
    ):
        """Make custom functions available to the template."""
        return {
            **context,
            'telegram': TELEGRAM,
        }

    def inference(self):
        logger.info('Marking Telegram links as such...')
        self.octiron.ldflex.graph.update(self.sparql_query)



from dataclasses import dataclass
from typing import Dict, Iterable, Optional

from iolanta.loaders.base import Loader, PyLDOptions, PyLDResponse
from iolanta.models import LDDocument, Quad
from rdflib import URIRef
from urlpath import URL


@dataclass(frozen=True)
class SchemeChoiceLoader(Loader):
    """Try to load a file via several loaders."""

    loader_by_scheme: Dict[str, Loader]

    def __call__(self, url: str, options: PyLDOptions) -> PyLDResponse:
        """Compile document for PyLD."""
        return {
            'document': self.as_jsonld_document(
                url=URL(url),
                iri=url,
            ),
            'contextUrl': None,
        }

    def resolve_loader_by_url(self, url: URL):
        """Find loader instance by URL."""
        try:
            return self.loader_by_scheme[url.scheme]
        except KeyError:
            raise ValueError(f'Cannot find a loader for URL: {url}')

    def as_jsonld_document(
        self,
        url: URL,
        iri: Optional[URIRef] = None,
    ) -> LDDocument:
        """Represent a file as a JSON-LD document."""
        return self.resolve_loader_by_url(
            url=url,
        ).as_jsonld_document(
            url=url,
            iri=iri,
        )

    def as_quad_stream(
        self,
        url: str,
        iri: Optional[URIRef],
        root_loader: Optional[Loader] = None,
    ) -> Iterable[Quad]:
        """Convert data into a stream of RDF quads."""
        return self.resolve_loader_by_url(
            url=url,
        ).as_quad_stream(
            url=url,
            iri=iri,
            root_loader=root_loader or self,
        )



import operator

from dominate.tags import ul, li

from iolanta.facet import Facet
from iolanta.namespaces import IOLANTA
from octadocs.iolanta import render
from octadocs_ibis.models import IBIS


class PositionsForIssue(Facet):
    """List positions to answer the Issue."""

    sparql_query = '''
    SELECT * WHERE {
        ?position ibis:responds-to $issue .
    } ORDER BY ?position
    '''

    def render(self):
        rows = self.query(self.sparql_query, issue=self.uriref)
        positions = list(map(operator.itemgetter('position'), rows))

        return ul(
            li(
                render(
                    position,
                    environments=[IOLANTA.html],
                    octiron=self.octiron,
                )
            ) for position in positions
        )



from typing import Optional

from mkdocs.theme import Theme


def language_from_config(config) -> Optional[str]:
    """Retrieve language from config."""
    theme: Theme = config['theme']
    return theme['language']




from dataclasses import dataclass
from typing import Any, Dict

from octadocs.pyld_document_loaders.base import DocumentLoader
from octadocs.pyld_document_loaders.models import JsonLDDocument
from pyld.jsonld import JsonLdError
from urlpath import URL


@dataclass
class NamedLoader(DocumentLoader):
    """
    Load JSON document from a named:// IRI.

    The mappings have been configured in mkdocs.yml, at extra.named section.
    """

    documents: Dict[str, Any]

    def __call__(self, url: str, options: Dict[str, Any]) -> JsonLDDocument:
        decoded_url = URL(url)

        if decoded_url.scheme == 'local':
            document_name = decoded_url.path

        else:
            raise ValueError('Scheme must be `local`.')

        try:
            response = {
                'document': self.documents[document_name],
                'contextUrl': None,
                'documentUrl': url,
                'contentType': 'application/ld+json',
            }
        except KeyError:
            raise JsonLdError(
                message=(
                    f'Cannot find a document under `{document_name}` label. '
                    f'Please check your mkdocs.yml configuration '
                    f'or plugin code.'
                ),
                type_='jsonld.InvalidUrl',
                details={
                    'url': url,
                    'named_contexts': list(self.documents.keys()),
                },
                code='loading document failed',
            )

        return response



from classes import typeclass
from rdflib import Graph, URIRef
from rdflib.term import Node

from octadocs.types import ComputedQName


@typeclass
def node_to_qname(node: Node, graph: Graph):
    """Convert a node to a QName."""


@node_to_qname.instance(object)
def _object_to_qname(node: URIRef, graph: Graph):
    return node


@node_to_qname.instance(URIRef)
def _uriref_to_qname(node: URIRef, graph: Graph):
    try:
        qname = graph.compute_qname(node)
    except ValueError:
        return node

    return ComputedQName(*qname)
